package cn.iocoder.yudao.module.shipment.utils;

import cn.iocoder.yudao.module.shipment.dto.LoadDto;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteTableHolder;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author huyufeng
 * @date 2022/12/14
 * @description 自定义单元格合并策略
 */

public class CustomLoadMergeStrategyNew extends AbstractMergeStrategy {

    /**
     * 分组，每几行合并一次
     */
    private List<Integer> exportFieldGroupCountList;

    private List<Integer> partMergeList;


    private int mergeType;

    /**
     * 目标合并列index
     */
    private Integer targetColumnIndex;

    // 需要开始合并单元格的首行index
    private Integer rowIndex;

    private List<Integer> colorRowList;

    private Map<String,Integer> colorMap;

    // exportDataList为待合并目标列的值   partMerge为另外一种合并方式计算好的合并行数，mergeType 合并类型  1为订单维度合并  2为入仓记录维度的合并
    public CustomLoadMergeStrategyNew(List<LoadDto> exportDataList, List<Integer> partMergeList, Integer targetColumnIndex, int mergeType,List<Integer> colorRowList, Map<String,Integer> colorMap) {
        this.exportFieldGroupCountList = getGroupCountList(exportDataList,targetColumnIndex);
        this.partMergeList = partMergeList;
        this.mergeType = mergeType;
        this.targetColumnIndex = targetColumnIndex;
        this.colorRowList = colorRowList;
        this.colorMap = colorMap;
    }

    @Override
    public void afterCellDispose(WriteSheetHolder writeSheetHolder, WriteTableHolder writeTableHolder, List<CellData> cellDataList, Cell cell, Head head, Integer relativeRowIndex, Boolean isHead) {
        Workbook workbook = writeSheetHolder.getSheet().getWorkbook();
        if (colorRowList.contains(cell.getRowIndex()) && cell.getColumnIndex() == targetColumnIndex) {
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setFillForegroundColor(IndexedColors.fromInt(colorMap.get(String.valueOf(cell.getRowIndex()))).index);
            cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            cell.setCellStyle(cellStyle);
        }
        if (!isHead) {
            this.merge(writeSheetHolder.getSheet(), cell, head, relativeRowIndex);
        }
    }

    @Override
    protected void merge(Sheet sheet, Cell cell, Head head, Integer relativeRowIndex) {

        if (null == rowIndex) {
            rowIndex = cell.getRowIndex();
        }

        // 仅从首行以及目标列的单元格开始合并，忽略其他
        if (cell.getRowIndex() == rowIndex && cell.getColumnIndex() == targetColumnIndex ) {
            if (mergeType == 1) {
                mergeGroupColumn(sheet,exportFieldGroupCountList);
            } else if (mergeType == 2) {
                mergeGroupColumn(sheet,partMergeList);
            }
        }
    }

    private void mergeGroupColumn(Sheet sheet,List<Integer> list) {
        int rowCount = rowIndex;
        for (Integer count : list) {
            if (count == 1) {
                rowCount += count;
                continue;
            }
            // 合并单元格
            CellRangeAddress cellRangeAddress = new CellRangeAddress(rowCount, rowCount + count - 1, targetColumnIndex, targetColumnIndex);
            sheet.addMergedRegionUnsafe(cellRangeAddress);
            rowCount += count;
        }
    }


    // 该方法将目标列根据值是否相同连续可合并，存储可合并的行数
    private List<Integer> getGroupCountList(List<LoadDto> exportDataList,Integer targetColumnIndex) {
        if (CollectionUtils.isEmpty(exportDataList)) {
            return new ArrayList<>();
        }

        List<Integer> groupCountList = new ArrayList<>();
        int count = 1;

        LinkedHashMap<String, List<LoadDto>> collect = exportDataList.stream().collect(Collectors.groupingBy(LoadDto::getId, LinkedHashMap::new, Collectors.toList()));
        for (Map.Entry<String, List<LoadDto>> entry : collect.entrySet()) {
            groupCountList.add(entry.getValue().size());
        }
        return groupCountList;
    }

}
